import * as fc from 'fast-check'
import formatPixelCount from './formatPixelCount'

it('should exist', () => {
  expect(formatPixelCount).toBeDefined()
})

it('should be a function', () => {
  expect(typeof formatPixelCount).toBe('function')
})

it('should take 1 argument', () => {
  expect(formatPixelCount).toHaveLength(1)
})

describe('on non-numeric arguments', () => {
  it('should throw an error', () => {
    fc.assert(
      fc.property(
        fc.anything().filter(v => typeof v !== 'number'),
        v => {
          expect(() => formatPixelCount(v as number)).toThrow(TypeError)
        }
      )
    )
  })
})

describe('on numeric arguments', () => {
  it('should throw an error on NaN', () => {
    expect(() => formatPixelCount(NaN)).toThrow(RangeError)
  })

  it('should format values of |x| < 1000', () => {
    fc.assert(
      fc.property(
        fc.integer().filter(v => Math.abs(v) < 1000),
        v => {
          expect(formatPixelCount(v)).toMatch(/^-?\d+ P$/)
        }
      )
    )
  })

  it('should format values of 1000 < |x| < 1000000', () => {
    fc.assert(
      fc.property(
        fc.integer().filter(v => 1000 < Math.abs(v) && Math.abs(v) < 1000000),
        v => {
          expect(formatPixelCount(v)).toMatch(/^-?\d+\.\d\d kP$/)
        }
      )
    )
  })

  it('should format values of |x| >= 1000000', () => {
    fc.assert(
      fc.property(
        fc.integer().filter(v => Math.abs(v) >= 1000000),
        v => {
          expect(formatPixelCount(v)).toMatch(/^-?\d+\.\d\d .P$/)
        }
      )
    )
  })
})
