import numeral from 'numeral'

type FormatPixelCount = (n: number) => string

const formatPixelCount: FormatPixelCount = n => {
  if (typeof n as unknown !== 'number') {
    throw TypeError('Argument should be a number.')
  }

  if (isNaN(n)) {
    throw RangeError('Cannot format NaN.')
  }

  const absValue = Math.abs(n)
  const theNumeral = numeral(n)

  if (absValue < 1000) {
    return `${absValue} P`
  }

  if (absValue < 1000000) {
    return `${theNumeral.format('0.00 a')}P`
  }

  return `${theNumeral.format('0.00 a').toUpperCase()}P`
}

export default formatPixelCount
