import * as fc from 'fast-check'
import { imageSize } from 'image-size'
import getDimensions, { Dimensions } from './getDimensions'

jest.mock('image-size')

it('should exist', () => {
  expect(getDimensions).toBeDefined()
})

it('should be a callable', () => {
  expect(typeof getDimensions).toBe('function')
})

it('should accept 1 argument', () => {
  expect(getDimensions).toHaveLength(1)
})

it('should throw an error given non-string arguments', async () => {
  await fc.assert(
    fc.asyncProperty(
      fc.anything().filter(v => typeof v !== 'string'),
      async notAString => {
        try {
          // expected to throw
          await getDimensions(notAString as string)
        } catch (err) {
          expect(err instanceof TypeError).toBe(true)
        }
      }
    )
  )
})

it('should throw an error given an empty string argument', async () => {
  try {
    await getDimensions('')
  } catch (err) {
    expect(err instanceof RangeError).toBe(true)
  }
})

describe('on string arguments', () => {
  beforeAll(() => {
    const imageSizeMock = imageSize as jest.Mock
    imageSizeMock.mockImplementation((_: string, cb: Function) => {
      cb(null, { images: [ { width: 0, height: 0, } ] })
    })
  })

  it('should return an array of objects containing `width` properties', async () => {
    await fc.assert(
      fc.asyncProperty(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        async aString => {
          const dimensionsArray: Dimensions[] = await getDimensions(aString)
          expect(
            dimensionsArray.every(d => 'width' in d)
          ).toBe(true)
        }
      )
    )
  })

  it('should return an array of objects containing `height` properties', async () => {
    await fc.assert(
      fc.asyncProperty(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        async aString => {
          const dimensionsArray: Dimensions[] = await getDimensions(aString)
          expect(
            dimensionsArray.every(d => 'height' in d)
          ).toBe(true)
        }
      )
    )
  })
})

describe('on error handling', () => {
  beforeAll(() => {
    const imageSizeMock = imageSize as jest.Mock
    imageSizeMock.mockImplementation((_: string, cb: Function) => {
      cb(new Error('vro.'))
    })
  })

  it('should throw caught error', async () => {
    await fc.assert(
      fc.asyncProperty(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        async aString => {
          try {
            await getDimensions(aString)
          } catch (err) {
            expect(err).toBeTruthy()
          }
        }
      )
    )
  })
})
