import { imageSize, } from 'image-size'

export type Dimensions = {
  width: number
  height: number
}

type GetDimensions = (src: string) => Promise<Dimensions[]>

const getDimensions: GetDimensions = async src => {
  if (typeof src as unknown !== 'string') {
    throw TypeError('Argument must be a string.')
  }

  if (src.trim().length < 1) {
    throw RangeError('Invalid value for argument.')
  }

  return new Promise((resolve, reject) => {
    imageSize(src, (err, dimensions) => {
      if (err) {
        reject(err)
        return
      }

      resolve(dimensions!.images as Dimensions[])
    })
  })
}

export default getDimensions
